	page 0
	cpu Z8601
	include	stddefZ8.inc

;*******************************************************************
;
;   Hilfsprogramme fr Ju-Te Tiny EMR-ES 4k
;   von Patrick Schfer, dr.p.schaefer@web.de
;
;   V1.01, 2012-04-12	SIO-Routinen hinzu
;   V1.02, 2012-04-16	Tor2-Transfer raus, Hauptmen hinzu
;
;*******************************************************************
;
; Aufruf aus dem EMR-ES mit 'P'rog, '1000', 'L'
;
; MemCopy:  kopiert einen Speicherbereich.
;   Start- und Zieladresse werden abgefragt. Fehler beim Schreiben
;   werden erkannt. Es darf auch vorwrts oder rckwrts berlappend
;   kopiert (d.h. verschoben) werden.
;
; SIO-Druck: Druckausgabe ber die serielle Schnittstelle
;   BASIC: Listing im Klartext. Anfang muss ber Bereich vorgegeben
;   werden (default %E000), Ende wird automatisch erkannt.
;   Maschinencode: Listing befehlsweise (wie PROG). Bereich muss
;   vorgegeben werden. 
;   Speicher: Listing als Hexdump, 16 Bytes pro Zeile. Bereich muss
;   vorgegeben werden. 
;   RAM-Editor: ruft die RAM- und Registeranzeige auf, 'shift'-'OFF'
;   zum Beenden.
;   Abbruch ist ber die Taste 'A' oder 'X' mglich.
; 
; SIO-Transfer: Speicherbereich ber die serielle Schnittstelle 
;   bertragen (Binr oder Intel-Hex)
;   Empfangen: binr (Bereich muss vorgegeben werden) oder Intel 
;   (Bereich wird automatisch erkannt)
;   Senden: Binr oder Intel, Bereich muss vorgegeben werden.
;   RAM-Editor: ruft die RAM- und Registeranzeige auf, 'shift'-'OFF'
;   zum Beenden.
;   Abbruch ist ber die Taste 'A' oder 'X' mglich.
;
; Datenformat 2400 bps, 8 bit, keine Paritt. Gesendet wird ber 
; P3.7, empfangen wird ber P3.0. Daher muss die Bilderzeugung auf 
; P3.5 umgelegt werden. Fr Tape In kann P3.1 verwendet werden.
; Lautsprecher und Tape Out hngen an P3.6. ber diesen Portpin
; werden zustzlich Statusinformationen beim Senden und Empfangen 
; von Daten ausgegeben.
; 
; Die Programme arbeiten in Registerbank %20.
;
;*******************************************************************

Version		SET "V1.02"

Baudrate	SET 2400		; 300 - 4800 bps sind mglich mit 8 MHz


; Einsprungadressen EMR-ES
;
ZBS		EQU 0827h		; Zeichen in %5A ausgeben an Cursorpos %5B
CLS		EQU 08DDh		; Bildschirm lschen
Begin		EQU 008FFh		; BASIC-Einsprung
PrtDezNr1	EQU 0AA3h		; Dezimalausgabe aus RR12 ohne fhrende Nullen (RP=%10 !)
NLE		EQU 0ACEh		; neue Zeile
TSK		EQU 0C1Dh		; auf Tastendruck warten, %5A
TAS		EQU 0C56h		; Tastenstatus prfen
PrtList		EQU 0DCCh		; Anzeige einer BASIC-Zeile ohne Zeilennummer ab Adresse aus %10-11
ES_Begin	EQU 2000h		; Rcksturz ins EMR-ES
ES_PrHex	EQU 20B6h		; Bytes aus %5D drucken
ES_PrMemWrd	EQU 20DBh		; Speicheradresse in %5E/5F drucken
ES_GetHexWrd	EQU 231Fh		; Hexadezimaleingabe eines Doppelbytes in %6E und %6F mit Anzeige, RP = #%60
ES_Data		EQU 2352h		; RAM- und Registeranzeige
ES_PrtPgmLn	EQU 260Eh		; Maschinenbefehl ab %5E/5F anzeigen


	org 0000h			; steht bei %0000 im EPROM
	phase 1000h			;  und luft spter bei %1000 im Speicher


;*******************************************************************
;
;  Tiny Toolkit
;  Hauptmen
;
;  arbeitet in Registerbank %20
;
;*******************************************************************

Toolkit		srp #20h
	assume RP:020h
Toolkit1	ld R14, #TK_Text_1 /256
		ld R15, #TK_Text_1 #256
		call PrintStr		; Hauptmen ausgeben
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'Z'		; Zurck?
		jr NZ, Toolkit2
		jp ES_Begin		;  EMR-ES neustarten

Toolkit2	cp 5Ah, #'M'		; Speicher kopieren
		jr NZ, Toolkit3
		call MemCopy
		jp Toolkit1

Toolkit3	cp 5Ah, #'D'		; Druckausgabe ber SIO
		jr NZ, Toolkit4
		call SioDruck
		jp Toolkit1

Toolkit4	cp 5Ah, #'S'		; Datenbertragung ber SIO
		jr NZ, Toolkit5
		call SioTransfer
		jp Toolkit1

Toolkit5	cp 5Ah, #'B'		; BASIC
		jp NZ, Toolkit1
		jp Begin
		jp Toolkit1


TK_Text_1	db 01, "TOOLKIT ", Version
		db 02, "M)EMCOPY"
		db 02, "D)RUCK (SIO)"
		db 02, "S)IO TRANSFER"
		db "B)ASIC"
		db 02, "Z)URUECK", 06



;*******************************************************************
;
;  PrintStr
;  basiert auf Code von E.Mueller 
;  http://hc-ddr.hucki.net/wiki/doku.php/tiny:emueller
;
; Text ab RR14 wird formatiert ausgeben, enthaltene Eingabebefehle
; werden interpretiert:
; 01 --> Bildschirm lschen
; 02 --> neue Zeile
; 03 --> Eingabe nach RR4 anfordern
; 04 --> Eingabe nach RR6 anfordern
; 05 --> Eingabe nach RR8 anfordern
; 06 --> Ende
;
; es wird in der aktuellen Registerbank gearbeitet, ndert R13
;
;*******************************************************************

PrintStr	ldc R13,@RR14		; ein Zeichen holen
		cp R13,#1		; 01 --> Bildschirm lschen
		jr NZ, PrStr1
		call CLS
		jr PrStr7

PrStr1		cp R13,#2		; 02 --> neue Zeile
		jr NZ, PrStr2
		call NLE
		jr PrStr7

PrStr2		cp R13,#3		; 03 --> Eingabe nach RR4 anfordern
		jr NZ, PrStr3
		call PrStr8
		ld R4, 06Eh
		ld R5, 06Fh
		jr PrStr7

PrStr3		cp R13,#4		; 04 --> Eingabe nach RR6 anfordern
		jr NZ, PrStr4
		call PrStr8
		ld R6, 06Eh
		ld R7, 06Fh
		jr PrStr7

PrStr4		cp R13,#5		; 05 --> Eingabe nach RR8 anfordern
		jr NZ, PrStr5
		call PrStr8
		ld R8, 06Eh
		ld R9, 06Fh
		jr PrStr7

PrStr5		cp R13,#6		; 06 --> Ende
		jr NZ,PrStr6
		ret

PrStr6		ld 05Ah, R13
		call ZBS		; Zeichen ausgeben
PrStr7		incw R14
		jr PrintStr

; Eingabe holen
PrStr8		ld R13, 05Bh		; Cursorposition zwischenspeichern
PrStr8a		ld 05Bh, R13
		push RP
		srp #60h
		call ES_GetHexWrd	; Doppelbyte holen
		pop RP
		jr C, PrStr8a		; Carry gesetzt --> Fehler, nochmal versuchen
		ret



;*******************************************************************
;
;  MemCopy
;  kopiert einen Speicherbereich. Es darf sowohl nach vorne als auch 
;  nach hinten berlappend kopiert (bzw verschoben) werden.
;
; Speicherverwendung
; %20/21	temporre Variablen beim Kopieren
; %24/25	Quelle Start
; %26/27	Quelle Ende
; %28/29	Ziel Start
; %2D		Zeichenpuffer bei Textausgabe
; %2E/2F	Zeiger auf Textstring
;
;*******************************************************************

MemCopy		ld R15,#MC_Text_1 #256
		ld R14,#MC_Text_1 /256
		call PrintStr		; Nach Quelle fragen und Eingaben vornehmen
		ld R1, R7
		ld R0, R6
		sub R1, R5		; Bereichsende grer als Anfang?
		sbc R0, R4
		jr C,MemCopy		;  --> neu versuchen!
		ld R15,#MC_Text_2 #256
		ld R14,#MC_Text_2 /256
		call PrintStr		; Nach Ziel fragen und Eingaben vornehmen
		call TSK		; auf Tastendruck warten
		cp 05Ah, #'J'		; 'J'?
		jr Z, MemCpy1		;  ja, weitermachen
		ret			;  sonst zurck

MemCpy1		di     			; Video aus
		ld R0, R8
		ld R1, R9		; Anfang Zielbereich
		sub R1, R5		;  minus Anfang Quellbereich
		sbc R0, R4
		jr C, MemCpy3		; --> wir kopieren nach vorne 

; hinten anfangen wenn Ziel hinter Quelle
		ld R1, R7
		ld R0, R6		; Quellende
		sub R1, R5		;  minus Quellanfang
		sbc R0, R4		; gibt Lnge
		add R9, R1		;  plus Zielanfang
		adc R8, R0		; gibt Zielende
		decw RR4		; Quellanfang -1
MemCpy4		ldc R0, @RR6		; Byte holen von %26/27
		lde @RR8, R0		; und nach %28/29 schreiben
		ldc R1, @RR8		; zurcklesen
		cp R1, R0		; gleich?
		jr NZ, MC_Fehler	;  nein, Fehler
		decw RR6		; Quelle runterzhlen
		decw RR8		; Ziel runterzhlen
		cp R6, R4		; Ende erreicht?
		jr NZ, MemCpy4		;  nein, dann weiter
		cp R7, R5
		jr NZ, MemCpy4

MC_Ende		ei     			; Video ein
		ld R15,#MC_Text_3 #256
		ld R14,#MC_Text_3 /256
		call PrintStr		; Erfolgsmeldung ausgeben
MC_Fehler1	ld R15,#MC_Text_6 #256
		ld R14,#MC_Text_6 /256
		call PrintStr		; nochmal j/n?
		call TSK		; auf Tastendruck warten
		cp 05Ah, #'J'		; 'J'?
		jr Z, MemCopy		;  ja, weitermachen
		ret			;  sonst zurck ins EMR-ES


; vorne anfangen wenn Ziel vor Quelle
MemCpy3		incw R6			; Quellende +1
MemCpy2		ldc R0, @RR4		; Byte holen von %24/25
		lde @RR8, R0		;  und nach %28/29 schreiben
		ldc R1, @RR8		; zurcklesen
		cp R1, R0		; gleich?
		jr NZ, MC_Fehler	;  nein, Fehler
		incw RR4		; Quelle hochzhlen
		incw RR8		; Ziel hochzhlen
		cp R6, R4		; Ende erreicht?
		jr NZ, MemCpy2		;  nein, dann weiter
		cp R7, R5
		jr NZ, MemCpy2
		jr MC_Ende


; Fehlerbehandlung
MC_Fehler	ei     			; Video ein
		ld R15,#MC_Text_4 #256
		ld R14,#MC_Text_4 /256
		call PrintStr		; Fehlermeldung drucken
		ld 5Eh, R8
       		ld 5Fh, R9		; Speicheradresse
    		call ES_PrMemWrd	; drucken
		jr MC_Fehler1


MC_Text_1	db 01, "  MEM-COPY"
		db 02, 02, "QUELLE"
		db 02, "VON: ", 03
		db 02, "BIS: ", 04, 06

MC_Text_2	db 02, "ZIEL"
		db 02, "NACH: ", 05
		db 02, "OK(J/N)?", 06

MC_Text_3	db 01, "  KOPIEREN"
		db 02, "   FERTIG!", 06

MC_Text_4	db 01, "FEHLER BEIM"
		db 02, "  SCHREIBEN!"
		db 02, 02, 06

MC_Text_6	db 02, 02, "NOCHMAL(J/N)?", 06



;*******************************************************************
;
;  SIO - Routinen
;
;  SioEnable	Umschalten von Bilderzeugung auf SIO,
;  SioDisable	Zurckschalten von SIO auf Bilderzeugung,
; 		SIO und Bild gleichzeitig geht nicht, da beide den
;		Timer T0 bentigen!
;  SioPutChar	gibt das Zeichen in R1 ber die SIO aus,
;  SioPutHex	gibt den Inhalt von R0 als Hexzahl aus. ndert R1,
;  SioCRLF	gibt CR+LF ber die SIO aus. ndert R1,.
;  SioHCB	gibt den Inhalt des Bildwiederholspeichers aus.
;               Leerzeichen am Ende werden ignoriert.
;		ndert R0, R1, RR6
;
;*******************************************************************

; SIO aktivieren, Video aus
;
SioEnable	di
		or P3, #080h		; TxD inaktiv setzen
		ld P3M, #041h		; keine Parity, P30+P37 fr SIO, P2 push-pull
		ld T0, #62500/Baudrate
		or TMR,#3
		or IRQ, #010h		; SIO bereit zum Senden
		and IRQ, #0F7h		; Empfangsregister leeren
		ret


; SIO deaktivieren, Bilderzeugung wieder einschalten
;
SioDisable	ld R1, #00Dh		; wir senden noch ein letztes Zeichen,
		call SioPutChar		;  was direkt abgewrgt wird weil wir den 
		ld P3M, #001h		; SIO ausmachen, sonst kommt kein Interrupt fr T0
		or P3, #080h
		ld T0, #64		; Timer 0 auf 64 (d.h. INT4 alle 64s bzw 15,625kHz)
		or TMR, #3		; Timer 0 laden und starten
		ei			; Bilderzeugung an
		ret


; Zeilenvorschub CR+LF ausgeben
;
SioCRLF		ld R1, #00Dh		; <CR>
		call SioPutChar		;  drucken
		ld R1, #00Ah		; <LF>


; ASCII-Zeichen in R1 ausgeben
; P3.6 ist high whrend auf die SIO gewartet wird
;
SioPutChar	or P3, #040h		; Laufzeitmessung: high = Warten
		tm IRQ, #010h		; warte auf SIO
		jr Z, SioPutChar
		ld SIO, R1 		; Byte ausgeben
		and P3, #0BFh
		clr IRQ 		; Interrupt quittieren
		ret


; R0 als Hexzahl ausgeben
; ndert R1
;
SioPutHex	call SioPutHex1
SioPutHex1	swap R0
		ld R1, R0
		and R1, #0Fh		; unteres Nibble freistellen
		cp R1, #10		; Ziffer?
		jr C, SioPutHex2
		add R1, #7		; sonst Korrektur
SioPutHex2	add R1, #030h		; ASCII erzeugen
		jp SioPutChar


; Inhalt des Bildspeichers %FD00-FD7F ausgeben
; Leerzeichen am Ende werden bersprungen
; (ndert R0, R1, RR6)
;
SioHCB		ld R6, #0FDh		; Ende des Bildwiederholspeichers
		ld R7, #080h
SioHC1		dec R7
		lde R1, @RR6		; ASCII-Zeichen holen
		cp R1, #' ' 		; Leerzeichen?
		jr Z, SioHC1		;   ja, berspringen
		ld R0, R7		; letzte verwendete Adresse
		clr R7			; erste verwendete Adresse
SioHC2		lde R1, @RR6		; ASCII-Zeichen holen
		call SioPutChar		;  und drucken
		call SioNBA		; nchste verwendete Speicheradresse
		cp R0, R7		; fertig?
		jr NC, SioHC2		;  nein, weiter
		ret

SioNBA		inc R7
		tcm R7, #13		; Zeilenende?
		jr NZ, SioNB1		;  nein
		add R7, #3		; Korrektur fr neue Zeile
SioNB1		ret


; prfen ob "A" oder "X" gedrckt wurde. Wenn ja dann Z=1.
; ndert R13
StopTaste	call TAS		; Taste gedrckt?
		ld R13, 06Dh
		and R13, #07Fh		; T-Flag maskieren
		cp R13, #'A'
		jr Z, StopTaste1	; Abbruch bei A
		cp R13, #'X'		; ... oder X
StopTaste1	clr 6Dh			; Tastendruck lschen
		ret



;*******************************************************************
;
;  SIO - Druck
;  Hardcopy eines BASIC-Listings, Maschinencode oder Hexdump
;  Bei MC und Hexdump ist Abbruch ber die Tasten 'A' und 'X'
;  mglich. Die Funktion basiert auf dem Konzept aus Hoyer - "Tiny",
;  Kapitel 10. D.h. die Daten werden erst in den Bildschirmspeicher
;  ausgeben, dann von dort als Hardcopy zur SIO geschickt.
;
;
; Speicherverwendung
; %20		temp
; %21		auszugebendes Zeichen
; %24/25	temp Anfang Speicherbereich
; %26/27	temp Ende Speicherbereich, Bildspeicher
; %28/29	Anfang Speicherbereich
; %2A/2B	Ende Speicherbereich
; %2D		Zeichenpuffer bei Textein-/ausgabe
; %2E/2F	Zeiger auf Textstring, Warteschleifenzhler
;
;*******************************************************************

SioDruck	or P3, #080h
		ld R8, #0E0h		; default Anfang
		ld R9, #000h
		ld R10, #0FBh		; default Ende
		ld R11, #0FFh
SioDruck1	ld R14, #SD_Text_1 /256
		ld R15, #SD_Text_1 #256
		call PrintStr		; Titel ausgeben
		call BereichDrucken
		ld R14, #SD_Text_2 /256
		ld R15, #SD_Text_2 #256
		call PrintStr
		ld R14, #SX_Text_3 /256
		ld R15, #SX_Text_3 #256
		call PrintStr		; restliches Men ausgeben
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'Z'		; Zurck?
		jr NZ, SioDruck2	;  nein, weiter
		ret			;  sonst beenden

SioDruck2	cp 5Ah, #'L'		; BASIC-Listing
		jr NZ, SioDruck3
		call SioBCY
		jp SioDruck1

SioDruck3	cp 5Ah, #'M'		; Maschinenprogramm-Listing
		jr NZ, SioDruck4
		call SioMCY
		jp SioDruck1

SioDruck4	cp 5Ah, #'S'		; Speicherauszug
		jr NZ, SioDruck5
		call SioHEX
		jp SioDruck1

SioDruck5	cp 5Ah, #'B'		; Bereich festlegen
		jp NZ, SioDruck6
		call BereichSetzen
		jp SioDruck1

SioDruck6	cp 5Ah, #'R'		; RAM und Registeranzeige aufrufen
		jp NZ, SioDruck1
		ld 06Eh, R8
		ld 06Fh, R9		; Startadresse setzen
		call ES_Data		;  und los!
		jp SioDruck1


SD_Text_1	db 01, "  SIO-DRUCK", 06

SD_Text_2	db 02, "L)IST BASIC"
		db 02, "M)CODE"
		db 02, "S)PEICHER", 06


; Speicherbereich abfragen
; RR8 Anfang, RR10 Ende
;
BereichSetzen	ld R14, #BD_Text_2 /256
		ld R15, #BD_Text_2 #256
		call PrintStr		; Text ausgeben und Werte holen
		ld R1, R7
		ld R0, R6
		sub R1, R5		; Bereichsende grer als Anfang?
		sbc R0, R4
		jr C,BereichSetzen	;  --> neu versuchen!
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'J'
		jr NZ, BereichS1	; Nein --> Abbruch
		ld R8, R4
		ld R9, R5		; sonst Bereich bernehmen
		ld R10, R6
		ld R11, R7
BereichS1	ret


; aktuellen Speicherbereich ausgeben
; RR8 Anfang, RR10 Ende
;
BereichDrucken	ld R14, #BD_Text_1 /256
		ld R15, #BD_Text_1 #256
		call PrintStr		; Bereich ausgeben
		ld 5Eh, R8
		ld 5Fh, R9		; Anfangsadresse
		call ES_PrMemWrd	; drucken
		ld 5Ah, #'-'
		call ZBS		; Strich drucken
		ld 5Eh, R10
		ld 5Fh, R11		; Endadresse
		call ES_PrMemWrd	; drucken
		ret


BD_Text_1	db 02, "B)EREICH", 02, "  ", 06

BD_Text_2	db 01, "BEREICH"
		db 02, "VON: ", 03
		db 02, "BIS: ", 04
		db 02, 02, "OK(J/N)?", 06



; Druck eines BASIC-Programms
; Das Programm wird ber die LIST-Routine zeilenweise in den Bild-
; wiederholspeicher ausgegeben und von dort zum SIO geschickt.
; RR8 enthlt die Startadresse des BASIC-Programms
;
SioBCY		call SioEnable
		ld 1Fh, #016h		; BASIC-Pointer
		ld 10h, R8		; Programmstart
		ld 11h, R9
		ld 6Eh, #00Ch		; LIST-Adresse
		ld 6Fh, #0E6h
		push RP
		call SioCRLF
SioBC1		call CLS		; Bild lschen
		srp #10h
		lde R2, @RR0		; Zeilennummer MSB
		or R2, R2		;  =0?
		jr Z, SioExit		;  dann fertig
		incw RR0
		lde R3, @RR0		; Zeilennummer LSB
		and R2, #07Fh		; Kennbit lschen
		call PrtDezNr1		; Zeilennummer in RR2 anzeigen
		ld 5Bh, #4		; Cursor auf Position 4
		call PrtList		; Zeile anzeigen
		jr NC, SioExit		; bei Fehler beenden
		call StopTaste		; Abbruch?
		jr Z, SioExit
		incw RR0		; sonst nchste Zeile
		pop RP
		push RP
		call SioHCB		; Bildschirminhalt drucken
		call SioCRLF
		jr SioBC1		; nchste Zeile

SioExit		call SioCRLF
		call SioCRLF
		pop RP
		call SioDisable
		ret


; Druck eines Maschinen-Programms
; Das Programm wird ber die entsprechnde Routine aus PROG zeilenweise
; in den Bildwiederholspeicher ausgegeben und von dort zum SIO geschickt.
; RR8 enthlt die Anfangs-, RR10 die Endadresse
;
SioMCY		call SioEnable
		ld 5Eh, R8		; Anfangsadresse
		ld 5Fh, R9
		ld R14, R10		; Endadresse
		ld R15, R11
		incw RR14		;  +1
		push RP
		call SioCRLF
SioMC1		srp #60h
		call CLS
		call ES_PrtPgmLn	; Maschinenbefehl ab %5E/5F anzeigen
		pop RP
		push RP
		call SioHCB		; Bildschirminhalt drucken
		call SioCRLF
		call StopTaste
		jr Z, SioExit
		ld R4, 5Eh
		ld R5, 5Fh		; aktuelle Adresse
		sub R5, R15		;  - Endadresse
		sbc R4, R14
		jr C, SioMC1		; <0 --> weiter

		jr SioExit


; Hexdump
; RR8 enthlt die Anfangs-, RR10 die Endadresse
;
SioHex		call SioEnable
		ld R4, R8		; Anfangsadresse
		ld R5, R9
		call SioCRLF
		push RP

SioHex4		ld R14, #16		; Zeilen pro Block
SioHex3		ld R15, #16		; Bytes pro Zeile
		call SioCRLF
		ld R0, R4		; Adresse MSB
		call SioPutHex
		ld R0, R5		; Adresse LSB
		call SioPutHex
		ld R1, #':'
		call SioPutChar
SioHex2		ld R1, #' '
		call SioPutChar
		ldc R0, @RR4		; Speicherinhalt
		call SioPutHex		;  drucken
		incw RR4		; nchstes Byte
		djnz R15, SioHex2
		ld R6, R10
		ld R7, R11		; Endadresse
		sub R7, R5		;  - aktuelle Adresse
		sbc R6, R4
		jr C, SioExit		; fertig?
		call StopTaste
		jr Z, SioExit
		djnz R14, SioHex3	; sonst nchste Zeile
		call SioCRLF		; Leerzeile
		jr SioHex4		; nchster Block



;*******************************************************************
;
;  SIO - Transfer
;  sendet oder empfngt einen Speicherblock als Binary oder Intel-Hex
;
;
; Speicherverwendung
; %20		zu sendendes / empfangenes Zeichen
; %21		Merker fr Ende-Kennung beim Empfang
; %24/25	temp Anfang Speicherbereich
; %26/27	temp Ende Speicherbereich
; %28/29	Anfang Speicherbereich
; %2A/2B	Ende Speicherbereich
; %2D		Zeichenpuffer bei Textein-/ausgabe
; %2E/2F	Zeiger auf Textstring, Warteschleifenzhler
;
;*******************************************************************


SioTransfer	or P3, #080h
		ld R8, #0E0h		; default Anfang
		ld R9, #000h
		ld R10, #0FBh		; default Ende
		ld R11, #0FFh
SioXfer1	ld R14, #SX_Text_1 /256
		ld R15, #SX_Text_1 #256
		call PrintStr		; Titel ausgeben
		call BereichDrucken
		ld R14, #SX_Text_2 /256
		ld R15, #SX_Text_2 #256
		call PrintStr		; restliches Men ausgeben
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'Z'		; Zurck?
		jr NZ, SioXfer2		;  nein, weiter
		ret			;  sonst beenden

SioXfer2	cp 5Ah, #'S'		; Senden
		jr NZ, SioXfer3
SioSend1	ld R14, #SX_Text_4 /256
		ld R15, #SX_Text_4 #256
		call PrintStr		; Datenformat abfragen
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'B'		; Binr?
		jr NZ, SioSend2		;  nein, weiter
		call SioSendBin		; binr senden
		jp SioXfer1

SioSend2	cp 5Ah, #'I'		; Intel?
		jr NZ, SioSend1		;  nein, neuer Versuch
		call SioSendIhex	; Intelhex senden
		jp SioXfer1

SioXfer3	cp 5Ah, #'E'		; Empfangen
		jr NZ, SioXfer4
SioEmpf1	ld R14, #SX_Text_4 /256
		ld R15, #SX_Text_4 #256
		call PrintStr		; Datenformat abfragen
		call TSK		; auf Tastendruck warten
		cp 5Ah, #'B'		; Binr?
		jr NZ, SioEmpf2		;  nein, weiter
		call SioEmpfBin		; binr empfangen
		jp SioXfer1

SioEmpf2	cp 5Ah, #'I'		; Intel?
		jr NZ, SioEmpf1		;  nein, neuer Versuch
		call SioEmpfIhex	; Intelhex empfangen
		jp SioXfer1

SioXfer4	cp 5Ah, #'B'		; Bereich festlegen
		jp NZ, SioXfer5
		call BereichSetzen
		jp SioXfer1

SioXfer5	cp 5Ah, #'R'		; RAM und Registeranzeige aufrufen
		jp NZ, SioXfer1
		ld 06Eh, R8
		ld 06Fh, R9		; Startadresse setzen
		call ES_Data		;  und los!
		jp SioXfer1


SX_Text_1	db 01, "SIO-TRANSFER", 02, 06

SX_Text_2	db 02, "E)MPFANGEN"
		db 02, "S)ENDEN"
SX_Text_3	db 02, "R)AM EDITOR"
		db 02, "Z)URUECK ", 06

SX_Text_4	db 01, 02, "I)NTEL ODER"
		db 02, "B)INARY", 06


; Speicherbereich als Intel-Hex senden
; Wir senden Typ 0-Record mit je 16 bytes, und als Abschluss einen Typ
; 1-Record mit gltiger Prfsumme. Das entspricht dem von p2hex erzeugten
; Format.
; RR8 enthlt die Anfangs-, RR10 die Endadresse
;
SioSendIhex	call SioEnable
		ld R4, R8		; Anfangsadresse
		ld R5, R9
SioSndIHex1	ld R2, #16		; default 16 bytes pro Record
		ld R6, R10
		ld R7, R11		; Endadresse
		incw RR6		;  +1
		sub R7, R5		;  - aktuelle Adresse
		sbc R6, R4		;  gibt noch zu sendende Bytes
		push R7
		push R6
		sub R7, #16		; reicht's fr einen Record?
		sbc R6, #0
		pop R6
		pop R7
		jr NC, SioSndIHex2	; ja
		ld R2, R7		;  sonst nur die Restmenge senden
SioSndIHex2	or R2, R2		; Restlnge Null?
		jr Z, SioSndIHex3	;  dann sind wir fertig	

		call SioCRLF
		ld R1, #':'		; Startzeichen
		call SioPutChar
		ld R0, R2		; Lnge
		ld R3, R2		; Prfsumme
		call SioPutHex
		ld R0, R4		; aktuelle Adresse
		add R3, R0		; Prfsumme hochzhlen
		call SioPutHex		
		ld R0, R5
		add R3, R0		; Prfsumme hochzhlen
		call SioPutHex
		ld R0, #0
		call SioPutHex

SioSndIHex4	ldc R0, @RR4		; Byte holen
		add R3, R0		; Prfsumme hochzhlen
		call SioPutHex		;  und senden
		incw RR4		; nchstes Byte
		djnz R2, SioSndIHex4	
		ld R0, R3		
		com R0
		inc R0			; Zweierkomplement der Prfsumme
		call SioPutHex
		call StopTaste		; Abbruch?
		jr Z, SioSndIHex5
		jr SioSndIHex1

SioSndIHex3	call SioCRLF
		ld R14, #IH_Text_1 /256
		ld R15, #IH_Text_1 #256
		call PrintStr		; Endrecord drucken
		call SioHCB		; und ausgeben
		call SioCRLF
SioSndIHex5	call SioCRLF
		call SioDisable
		ret

IH_Text_1	db 01, ":00000001FF", 06


; Intel Hex-File empfangen
; Es werden Typ 0-Records zwischen 1 und 255 bytes Lnge verarbeitet. Alles 
; ausserhalb ':' und der Prfsumme wird ignoriert. Die im Record enthaltene
; Zieladresse wird verwendet. Abbruch erfolgt, wenn ein Typ 1-Record
; empfangen oder die Abbruchtaste gedrckt wurde.
;
; R0:	empfangenes Zeichen
; R1:	Lnge des Records 
; R2: 	temp Datenbyte
; R3:	temp Prfsumme
; RR4:	Zieladresse
; R6:	Fehlerflag. 01= Prfsummenfehler, 02= Datenfehler, 
;	04= falscher Recordtyp
;
; P3.6 ist high whrend auf ein Record empfangen wird.
;
SioEmpfIhex	call SioEnable
		clr R6			; Fehlerflag lschen
SioEmpfIhex1	clr R3			; Prfsumme lschen
		and P3, #0BFh
		call SioGetByte
		cp R0, #':'		; Startzeichen?
		jr NZ, SioEmpfIhex1	; sonst ignorieren

		or P3, #040h
		call SioGetByte
		call CharNachHex
		jp NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R1, R0		; Lnge, obere Stelle
		call SioGetByte
		call CharNachHex
		jp NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R1, R0		; untere Stelle hinzufgen, Lnge in R1
		add R3, R1		; Prfsumme aktualisieren

		call SioGetByte
		call CharNachHex
		jp NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R4, R0		; Adresse MSB, obere Stelle
		call SioGetByte
		call CharNachHex
		jp NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R4, R0		; untere Stelle hinzufgen, Adresse MSB in R4
		add R3, R4		; Prfsumme aktualisieren

		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R5, R0		; Adresse LSB, obere Stelle
		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R5, R0		; untere Stelle hinzufgen, Adresse LSB in R5
		add R3, R5		; Prfsumme aktualisieren

		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R2, R0		; Typ, obere Stelle
		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R2, R0		; untere Stelle hinzufgen, Typ in R2
		add R3, R2		; Prfsumme aktualisieren

		cp R2, #1		; Typ 1 - Record?
		jr Z, SioEmpfIdone	;  dann sind wir fertig!
		cp R2, #0		; Typ 0 - Record?
		jr Z, SioEmpfIhex2	;  ja, Daten empfangen
		or R6, #4		; nicht untersttzter Recordtyp
		jp SioEmpfIhex1

SioEmpfIhex2	call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R2, R0		; Datenbyte, obere Stelle
		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R2, R0		; untere Stelle hinzufgen
		add R3, R2		; Prfsumme aktualisieren
		lde @RR4, R2		;  und Datenbyte einsortieren
		incw RR4
		djnz R1, SioEmpfIhex2
		
		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		swap R0
		ld R2, R0		; Prfbyte, obere Stelle
		call SioGetByte
		call CharNachHex
		jr NC, SioEmpfIerr	; ungltiges Zeichen, Record ignorieren
		or R2, R0		; untere Stelle hinzufgen
		add R3, R2		; Prfsumme aktualisieren
		jp Z, SioEmpfIhex1	; ok, nchster Record

		or R6, #1		; Prfsummenfehler
		jp SioEmpfIhex1

SioEmpfIerr	or R6, #2		; Datenfehler
		jp SioEmpfIhex1		; neuen Record abwarten

SioEmpfIdone	call SioDisable
		or R6, R6		; alle ok?
		jr Z, SioEmpfIdn1
		ld R14, #IH_Text_3 /256
		ld R15, #IH_Text_3 #256
		call PrintStr		; "Fehler"
		ld 5Dh, R6
		call ES_PrHex
		call NLE
		jr SioEmpfIdn2
SioEmpfIdn1	ld R14, #IH_Text_2 /256
		ld R15, #IH_Text_2 #256
		call PrintStr		; "ok"
SioEmpfIdn2	call TSK		; auf Tastendruck warten
		ret


IH_Text_2	db 01, "OK", 06

IH_Text_3	db 01, "FEHLER: ", 06


; Speicherbereich binr senden
; RR8 enthlt die Anfangs-, RR10 die Endadresse
; ndert R1, RR4, RR6
;
SioSendBin	call SioEnable
		ld R4, R8		; Anfangsadresse
		ld R5, R9
SioSndBin1	ldc R1, @RR4		; Byte holen
		call SioPutChar		;  und senden
		incw RR4		; nchstes Byte
		ld R6, R10
		ld R7, R11		; Endadresse
		sub R7, R5		;  - aktuelle Adresse
		sbc R6, R4
		jr C, SioSndBin2	; fertig?
		call StopTaste		; Abbruch?
		jr Z, SioSndBin2
		jr SioSndBin1		; sonst weitermachen
SioSndBin2	call SioDisable
		ret


; Speicherbereich binr empfangen
; bis Bereich voll oder Abbruchtaste gedrckt
; RR8 enthlt die Anfangs-, RR10 die Endadresse
; ndert R0, RR4, RR6
;
SioEmpfBin	call SioEnable
		ld R4, R8		; Anfangsadresse
		ld R5, R9
SioEmpfBin1	call SioGetByte
		or P3, #040h
		lde @RR4, R0		;  und einrumen
		incw RR4
		ld R6, R10
		ld R7, R11		; Endadresse
		sub R7, R5		;  - aktuelle Adresse
		sbc R6, R4
		and P3, #0BFh
		jr NC, SioEmpfBin1	; fertig?
SioEmpfBExit	call SioDisable
		ret

; auf ein Byte von der SIO warten, Ergebnis in R0 zurckliefern
; Abbruch wenn Taste 'A' oder 'X' gedrckt
;
SioGetByte	call StopTaste		; Abbruch?
		jr NZ, SioGetByte1
		pop R0			; Rcksprungadresse von SioGetByte
		pop R0			;  entfernen
		jr SioEmpfBExit
SioGetByte1	tm IRQ, #008h		; schon was da?
		jr Z, SioGetByte
		ld R0, SIO		; Byte abholen,
		and IRQ, #0F7h		;  besttigen
		ret


; wandelt die den Inhalt von R0 in eine Hexzahl um
; gltiger Bereich '0'..'9' = %30..39, 'A'..'F' = %41..46, 
; 'a'..'f' = %61..66. 
; Carry gesetzt wenn ok
;
CharNachHex	cp R0, #060h
		jr C, CharNachHex1	; --> kein Kleinbuchstabe
		sub R0, #87		; 'a'..'f' -> 10..15
		jr CharNachHex3

CharNachHex1 	cp R0, #040h		
		jr C, CharNachHex2	; --> kein Grobuchstabe
		sub R0, #55		; 'A'..'F' -> 10..15
		jr CharNachHex3

CharNachHex2	sub R0, #030h		; '0'..'9' -> 0..9

CharNachHex3	cp R0, #16
		ret			; C=1 wenn ok

	
	db Toolkit+0800h-$ dup(0FFh)	; mit %FF auffllen

	end

